/**************************************************************************
 * fg_random.c -- routines to handle random number generation
 *
 * Written by Curtis Olson, started July 1997.
 *
 * Copyright (C) 1997  Curtis L. Olson  - curt@infoplane.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: fg_random.c,v 1.1 1997/07/30 16:04:09 curt Exp $
 * (Log is kept at end of this file)
 **************************************************************************/


#include <stdlib.h>         /* for random(), srandom() */
#include <time.h>           /* for time() to seed srandom() */        

#include "fg_random.h"

#ifndef USE_RAND
#  ifdef sgi
#    undef RAND_MAX
#    define RAND_MAX 2147483647
#  endif
#endif


/* Seed the random number generater with time() so we don't see the
 * same sequence every time */
void fg_srandom(void) {
#ifdef USE_RAND
    srand(time(NULL));
#else
    srandom(time(NULL));
#endif                                       
}


/* return a random number between [0.0, 1.0) */
double fg_random(void) {
#ifdef USE_RAND
    return(rand() / (double)RAND_MAX);
#else
    return(random() / (double)RAND_MAX);
#endif
}


/* $Log: fg_random.c,v $
 * Revision 1.1  1997/07/30 16:04:09  curt
 * Moved random routines from Utils/ to Math/
 *
 * Revision 1.1  1997/07/19 22:31:57  curt
 * Initial revision.
 *
 */
